init_config:
  process_fs_path: /rootfs/proc
instances:
{% if MONASCA_MONITORING %}
- name: influxd
  detailed: true
  dimensions:
    service: influxd
  exact_match: false
  search_string:
  - influxd
- name: monasca-statsd
  detailed: true
  dimensions:
    service: monasca-statsd
  exact_match: false
  search_string:
  - monasca-statsd
- name: monasca-notification
  detailed: true
  dimensions:
    service: monasca-notification
  exact_match: false
  search_string:
  - monasca-notification
- name: persister
  detailed: true
  dimensions:
    service: persister
  exact_match: false
  search_string:
  - persister
- name: monasca-thresh
  detailed: true
  dimensions:
    service: monasca-thresh
  exact_match: false
  search_string:
  - monasca-thresh
- name: monasca-api
  detailed: true
  dimensions:
    service: monasca-api
  exact_match: false
  search_string:
  - monasca-api
- name: monasca-collector
  detailed: true
  dimensions:
    service: monasca-collector
  exact_match: false
  search_string:
  - monasca-collector
- name: memcached
  detailed: true
  dimensions:
    service: memcached
  exact_match: false
  search_string:
  - memcached
- name: cadvisor
  detailed: true
  dimensions:
    service: cadvisor
  exact_match: false
  search_string:
  - cadvisor
- name: monasca-forwarder
  detailed: true
  dimensions:
    service: monasca-forwarder
  exact_match: false
  search_string:
  - monasca-forwarder
- name: zookeeper
  detailed: true
  dimensions:
    service: zookeeper
  exact_match: false
  search_string:
  - zookeeper
- name: kafka
  detailed: true
  dimensions:
    service: kafka
  exact_match: false
  search_string:
  - kafka
- name: mysqld
  detailed: true
  dimensions:
    service: mysqld
  exact_match: false
  search_string:
  - mysqld
{% endif %}
{% if MONASCA_LOG_MONITORING %}
- name: logspout
  detailed: true
  dimensions:
    service: logspout
  exact_match: false
  search_string:
  - logspout
- name: log-agent
  detailed: true
  dimensions:
    service: log-agent
  exact_match: false
  search_string:
  - log-agent
- name: log-api
  detailed: true
  dimensions:
    service: log-api
  exact_match: false
  search_string:
  - log-api
- name: kibana
  detailed: true
  dimensions:
    service: kibana
  exact_match: false
  search_string:
  - kibana
- name: elasticsearch
  detailed: true
  dimensions:
    service: elasticsearch
  exact_match: false
  search_string:
  - elasticsearch
- name: log-transformer
  detailed: true
  dimensions:
    service: log-transformer
  exact_match: false
  search_string:
  - log-transformer
- name: log-persister
  detailed: true
  dimensions:
    service: log-persister
  exact_match: false
  search_string:
  - log-persister
- name: log-metrics
  detailed: true
  dimensions:
    service: log-metrics
  exact_match: false
  search_string:
  - log-metrics
{% endif %}
