Api:
  url: "{{ MONASCA_URL | default('') }}"
  service_type: "{{ SERVICE_TYPE | default('') }}"
  endpoint_type: "{{ ENDPOINT_TYPE | default('') }}"
  region_name: "{{ REGION_NAME | default('') }}"

  username: "{{ OS_USERNAME }}"
  password: "{{ OS_PASSWORD }}"
  keystone_url: "{{ OS_AUTH_URL }}"
  user_domain_id: "{{ OS_USER_DOMAIN_ID | default('') }}"
  user_domain_name: "{{ OS_USER_DOMAIN_NAME | default('') }}"
  project_name: "{{ OS_PROJECT_NAME | default('') }}"
  project_domain_id: "{{ OS_PROJECT_DOMAIN_ID | default('') }}"
  project_domain_name: "{{ OS_PROJECT_DOMAIN_NAME | default('') }}"
  project_id: "{{ OS_PROJECT_ID | default('') }}"
  insecure: {{ INSECURE | default(False)}}
  ca_file: "{{ CA_FILE | default('') }}"

  max_buffer_size: {{ MAX_BUFFER_SIZE | default(1000) }}
  max_batch_size: {{ MAX_BATCH_SIZE | default(0) }}
  max_measurement_buffer_size: {{ MAX_MEASUREMENT_BUFFER_SIZE | default(-1)}}
  backlog_send_rate: {{ BACKLOG_SEND_RATE | default(5) }}

Main:
  hostname: {{ '"%s"' % AGENT_HOSTNAME if AGENT_HOSTNAME else 'null' }}
{% if DIMENSIONS %}
  dimensions:
  {% for dimension in DIMENSIONS.split(',') %}
    {% set k, v = dimension.split('=', 1) %}
    {{k.strip()}}: "{{v}}"
  {% endfor %}
{% else %}
  dimensions: {}
{% endif %}
  check_freq: {{ CHECK_FREQ | default(30) }}
  num_collector_threads: {{ NUM_COLLECTOR_THREADS | default(1) }}
  pool_full_max_retries: {{ POOL_FULL_MAX_TRIES | default(4) }}
  sub_collection_warn: {{ SUB_COLLECTION_WARN | default(6) }}
  autorestart: {{ AUTORESTART | default(False) }}
  collector_restart_interval: {{ COLLECTOR_RESTART_INTERVAL | default(24) }}
  non_local_traffic: {{ NON_LOCAL_TRAFFIC | default(False) }}
  forwarder_url: {{ FORWARDER_URL | default("http://localhost:17123") }}

Statsd:
  monasca_statsd_port : {{ STATSD_PORT | default(8125) }}

Logging:
  log_level: {{ LOG_LEVEL | default('WARN') }}
  disable_file_logging: True
