# (C) Copyright 2017 Hewlett Packard Enterprise Development LP

init_config:
  connection_timeout: {{ KUBERNETES_API_TIMEOUT | default(3) }}

instances:
  {% if KUBERNETES_API_HOST %}
  - host: "{{ KUBERNETES_API_HOST }}"
    kubernetes_master_port: {{ KUBERNETES_API_PORT | default(8080) }}
  {% else %}
  - derive_api_url: true
  {% endif %}
    {% if KUBERNETES_API_CUSTOM_LABELS %}
    custom_kubernetes_labels:
      {% for label in KUBERNETES_API_CUSTOM_LABELS.split(',') %}
      - "{{ label }}"
      {% endfor %}
    {% endif %}
    {% if KUBERNETES_NAMESPACE_ANNOTATIONS %}
    namespace_annotations:
      {% for annotation in KUBERNETES_NAMESPACE_ANNOTATIONS.split(',') %}
      - "{{ annotation }}"
      {% endfor %}
    {% endif %}
    {% if STORAGE_PARAMETERS_DIMENSIONS %}
    storage_parameters_dimensions:
      {% for parameter in STORAGE_PARAMETERS_DIMENSIONS.split(',') %}
      - "{{ parameter }}"
      {% endfor %}
    {% endif %}
    report_persistent_storage: {{ REPORT_PERSISTENT_STORAGE | default(true) }}
