# (C) Copyright 2017 Hewlett Packard Enterprise Development LP

init_config:
  connection_timeout: {{ CADVISOR_TIMEOUT | default(3) }}
  {% if CADVISOR_MINIMUM_WHITELIST %}
  white_list:
    metrics:
      cpu.total_time_sec:
      mem.cache_bytes:
      mem.swap_bytes:
      mem.used_bytes:
      mem.working_set_bytes:
  {% endif %}
instances:
  {% if CADVISOR_URL %}
  - cadvisor_url: "{{ CADVISOR_URL }}"
  {% else %}
  - kubernetes_detect_cadvisor: True
  {% endif %}
