# (C) Copyright 2017 Hewlett Packard Enterprise Development LP

init_config:
  connection_timeout: {{ KUBERNETES_TIMEOUT | default(3) }}
  {% if KUBERNETES_MINIMUM_WHITELIST %}
  white_list:
    metrics:
      pod.cpu.total_time_sec:
      pod.mem.cache_bytes:
      pod.mem.swap_bytes:
      pod.mem.used_bytes:
      pod.mem.working_set_bytes:
      pod.mem.rss_bytes:
      container.mem.usage_percent:
  {% endif %}
  report_container_mem_percent: True
instances:
  - derive_host: True
    {% if KUBERNETES_NAMESPACE_ANNOTATIONS %}
    namespace_annotations:
      {% for annotation in KUBERNETES_NAMESPACE_ANNOTATIONS.split(',') %}
      - "{{ annotation }}"
      {% endfor %}
    {% endif %}