init_config:

instances:
- built_by: Kafka
  consumer_groups:
    1_metrics:
      metrics: []
    thresh-event:
      events: []
    thresh-metric:
      metrics: []
  kafka_connect_str: "{{ KAFKA_CONNECT_STR | default('kafka:9092') }}"
  name: "{{ KAFKA_CONNECT_STR | default('kafka:9092') }}"
  per_partition: false
